@@ -22,7 +22,7 @@
 #include <gtk/gtk.h>
 #include <gtk/gtkwidget.h>
 #include <sys/types.h>
-#include <util/gailmisc.h>
+#include <libgail-util/gailmisc.h>
 #include <pango/pango-layout.h>
 #include <atk/atkobject.h>
 
@@ -34,6 +34,7 @@
 static void pango_accessible_class_init       (PangoAccessibleClass *klass);
 
 static gint pango_accessible_get_n_children   (AtkObject       *obj);
+static gint pango_accessible_get_index_in_parent   (AtkObject       *obj);
 
 static void pango_accessible_real_initialize  (AtkObject *obj, gpointer data);
 static void pango_accessible_finalize         (GObject        *object);
@@ -79,6 +80,7 @@ static gint pango_accessible_get_offset_at_point (AtkText      *text,
 						  gint         y,
 						  AtkCoordType coords);
 static AtkAttributeSet* pango_accessible_get_default_attributes (AtkText *text);
+static gint pango_accessible_get_caret_offset (AtkText *text);
 
 
 /* AtkComponent Interfaces */
@@ -135,7 +137,7 @@ pango_accessible_get_type (void)
 			NULL
 		};
 
-		type = g_type_register_static (GTK_TYPE_ACCESSIBLE,
+		type = g_type_register_static (ATK_TYPE_OBJECT,
                                    	       "PangoAccessible", &tinfo, 0);
 		g_type_add_interface_static(type, ATK_TYPE_TEXT,
 					    &atk_text_info);
@@ -159,6 +161,7 @@ pango_accessible_class_init (PangoAccessibleClass *klass)
 	gobject_class->finalize = pango_accessible_finalize;
 
 	class->get_n_children = pango_accessible_get_n_children;
+	class->get_index_in_parent = pango_accessible_get_index_in_parent;
 	class->initialize = pango_accessible_real_initialize;
 }
 
@@ -209,6 +212,18 @@ pango_accessible_get_n_children (AtkObject* obj)
 	return 0;
 }
 
+static gint
+pango_accessible_get_index_in_parent (AtkObject* obj)
+{
+	gint i;
+
+	for (i = 0; i < CD_DISPLAY_END; i++) {
+		if (pango_accessible [i] == obj)
+			return i;
+	}
+	return -1;
+}
+
 static void
 pango_accessible_finalize (GObject *object)
 {
@@ -235,7 +250,7 @@ pango_accessible_get_text (AtkText *text,
 				  pango_layout_get_text(playout));
 
 	return gail_text_util_get_substring (PANGO_ACCESSIBLE(text)->textutil,
-					     &start_pos, &end_pos);
+					     start_pos, end_pos);
 }
 
 static gchar*
@@ -482,6 +497,12 @@ pango_accessible_get_default_attributes (AtkText *text)
 	return at_set;
 }
 
+static gint
+pango_accessible_get_caret_offset (AtkText *text)
+{
+	return 0;
+}
+
 /*
  * selection and caret related functions are not needed for PangoLayout.
  */
@@ -501,6 +522,7 @@ atk_text_interface_init (AtkTextIface *iface)
 	iface->get_default_attributes = pango_accessible_get_default_attributes;
 	iface->get_offset_at_point = pango_accessible_get_offset_at_point;
 	iface->get_character_extents = pango_accessible_get_character_extents;
+	iface->get_caret_offset = pango_accessible_get_caret_offset;
 }
 
 static void
